<?php
/**
 * @copyright 2010-2015 JTL-Software GmbH
 * @package Jtl\Connector\Core\Model
 * @subpackage Product
 */

namespace Jtl\Connector\Core\Model;

use JMS\Serializer\Annotation as Serializer;

/**
 * Localized category properties. localeName, categoryId and a localized name must be set.
 *
 * @access public
 * @package Jtl\Connector\Core\Model
 * @subpackage Product
 * @Serializer\AccessType("public_method")
 */
class CategoryI18n extends AbstractI18n
{
    /**
     * @var string Optional localized Long Description
     * @Serializer\Type("string")
     * @Serializer\SerializedName("description")
     * @Serializer\Accessor(getter="getDescription",setter="setDescription")
     */
    protected $description = '';

    /**
     * @var string Optional localized  short description used for meta tag description
     * @Serializer\Type("string")
     * @Serializer\SerializedName("metaDescription")
     * @Serializer\Accessor(getter="getMetaDescription",setter="setMetaDescription")
     */
    protected $metaDescription = '';
    
    /**
     * @var string Optional localized meta tag keywords value
     * @Serializer\Type("string")
     * @Serializer\SerializedName("metaKeywords")
     * @Serializer\Accessor(getter="getMetaKeywords",setter="setMetaKeywords")
     */
    protected $metaKeywords = '';
    
    /**
     * @var string Localized category name
     * @Serializer\Type("string")
     * @Serializer\SerializedName("name")
     * @Serializer\Accessor(getter="getName",setter="setName")
     */
    protected $name = '';
    
    /**
     * @var string Optional localized title tag value
     * @Serializer\Type("string")
     * @Serializer\SerializedName("titleTag")
     * @Serializer\Accessor(getter="getTitleTag",setter="setTitleTag")
     */
    protected $titleTag = '';
    
    /**
     * @var string Optional localized category URL
     * @Serializer\Type("string")
     * @Serializer\SerializedName("urlPath")
     * @Serializer\Accessor(getter="getUrlPath",setter="setUrlPath")
     */
    protected $urlPath = '';

    /**
     * @param string $description Optional localized Long Description
     * @return CategoryI18n
     */
    public function setDescription(string $description): CategoryI18n
    {
        $this->description = $description;
        
        return $this;
    }
    
    /**
     * @return string Optional localized Long Description
     */
    public function getDescription(): string
    {
        return $this->description;
    }
    
    /**
     * @param string $metaDescription Optional localized  short description used for meta tag description
     * @return CategoryI18n
     */
    public function setMetaDescription(string $metaDescription): CategoryI18n
    {
        $this->metaDescription = $metaDescription;
        
        return $this;
    }
    
    /**
     * @return string Optional localized  short description used for meta tag description
     */
    public function getMetaDescription(): string
    {
        return $this->metaDescription;
    }
    
    /**
     * @param string $metaKeywords Optional localized meta tag keywords value
     * @return CategoryI18n
     */
    public function setMetaKeywords(string $metaKeywords): CategoryI18n
    {
        $this->metaKeywords = $metaKeywords;
        
        return $this;
    }
    
    /**
     * @return string Optional localized meta tag keywords value
     */
    public function getMetaKeywords(): string
    {
        return $this->metaKeywords;
    }
    
    /**
     * @param string $name Localized category name
     * @return CategoryI18n
     */
    public function setName(string $name): CategoryI18n
    {
        $this->name = $name;
        
        return $this;
    }
    
    /**
     * @return string Localized category name
     */
    public function getName(): string
    {
        return $this->name;
    }
    
    /**
     * @param string $titleTag Optional localized title tag value
     * @return CategoryI18n
     */
    public function setTitleTag(string $titleTag): CategoryI18n
    {
        $this->titleTag = $titleTag;
        
        return $this;
    }
    
    /**
     * @return string Optional localized title tag value
     */
    public function getTitleTag(): string
    {
        return $this->titleTag;
    }
    
    /**
     * @param string $urlPath Optional localized category URL
     * @return CategoryI18n
     */
    public function setUrlPath(string $urlPath): CategoryI18n
    {
        $this->urlPath = $urlPath;
        
        return $this;
    }
    
    /**
     * @return string Optional localized category URL
     */
    public function getUrlPath(): string
    {
        return $this->urlPath;
    }
}
