<?php
/**
 * @copyright 2010-2015 JTL-Software GmbH
 * @package Jtl\Connector\Core\Model
 * @subpackage Product
 */

namespace Jtl\Connector\Core\Model;

use JMS\Serializer\Annotation as Serializer;

/**
 * customer order item variation
 *
 * @access public
 * @package Jtl\Connector\Core\Model
 * @subpackage Product
 * @Serializer\AccessType("public_method")
 */
class CustomerOrderItemVariation extends AbstractIdentity
{
    /**
     * @var Identity Reference to productVariation
     * @Serializer\Type("Jtl\Connector\Core\Model\Identity")
     * @Serializer\SerializedName("productVariationId")
     * @Serializer\Accessor(getter="getProductVariationId",setter="setProductVariationId")
     */
    protected $productVariationId = null;
    
    /**
     * @var Identity Reference to productVariationValue
     * @Serializer\Type("Jtl\Connector\Core\Model\Identity")
     * @Serializer\SerializedName("productVariationValueId")
     * @Serializer\Accessor(getter="getProductVariationValueId",setter="setProductVariationValueId")
     */
    protected $productVariationValueId = null;
    
    /**
     * @var string Optional custom text value for variation
     * @Serializer\Type("string")
     * @Serializer\SerializedName("freeField")
     * @Serializer\Accessor(getter="getFreeField",setter="setFreeField")
     */
    protected $freeField = '';
    
    /**
     * @var string Variation name e.g. 'color'
     * @Serializer\Type("string")
     * @Serializer\SerializedName("productVariationName")
     * @Serializer\Accessor(getter="getProductVariationName",setter="setProductVariationName")
     */
    protected $productVariationName = '';
    
    /**
     * @var double
     * @Serializer\Type("double")
     * @Serializer\SerializedName("surcharge")
     * @Serializer\Accessor(getter="getSurcharge",setter="setSurcharge")
     */
    protected $surcharge = 0.0;
    
    /**
     * @var string
     * @Serializer\Type("string")
     * @Serializer\SerializedName("valueName")
     * @Serializer\Accessor(getter="getValueName",setter="setValueName")
     */
    protected $valueName = '';

    /**
     * Constructor.
     * @param string $endpoint
     * @param int $host
     */
    public function __construct(string $endpoint = '', int $host = 0)
    {
        parent::__construct($endpoint, $host);
        $this->productVariationValueId = new Identity();
        $this->productVariationId = new Identity();
    }

    /**
     * @param Identity $productVariationId Reference to productVariation
     * @return CustomerOrderItemVariation
     */
    public function setProductVariationId(Identity $productVariationId): CustomerOrderItemVariation
    {
        $this->productVariationId = $productVariationId;
        
        return $this;
    }
    
    /**
     * @return Identity Reference to productVariation
     */
    public function getProductVariationId(): Identity
    {
        return $this->productVariationId;
    }
    
    /**
     * @param Identity $productVariationValueId Reference to productVariationValue
     * @return CustomerOrderItemVariation
     */
    public function setProductVariationValueId(Identity $productVariationValueId): CustomerOrderItemVariation
    {
        $this->productVariationValueId = $productVariationValueId;
        
        return $this;
    }
    
    /**
     * @return Identity Reference to productVariationValue
     */
    public function getProductVariationValueId(): Identity
    {
        return $this->productVariationValueId;
    }
    
    /**
     * @param string $freeField Optional custom text value for variation
     * @return CustomerOrderItemVariation
     */
    public function setFreeField(string $freeField): CustomerOrderItemVariation
    {
        $this->freeField = $freeField;
        
        return $this;
    }
    
    /**
     * @return string Optional custom text value for variation
     */
    public function getFreeField(): string
    {
        return $this->freeField;
    }
    
    /**
     * @param string $productVariationName Variation name e.g. 'color'
     * @return CustomerOrderItemVariation
     */
    public function setProductVariationName(string $productVariationName): CustomerOrderItemVariation
    {
        $this->productVariationName = $productVariationName;
        
        return $this;
    }
    
    /**
     * @return string Variation name e.g. 'color'
     */
    public function getProductVariationName(): string
    {
        return $this->productVariationName;
    }
    
    /**
     * @param double $surcharge
     * @return CustomerOrderItemVariation
     */
    public function setSurcharge(float $surcharge): CustomerOrderItemVariation
    {
        $this->surcharge = $surcharge;
        
        return $this;
    }
    
    /**
     * @return double
     */
    public function getSurcharge(): float
    {
        return $this->surcharge;
    }
    
    /**
     * @param string $valueName
     * @return CustomerOrderItemVariation
     */
    public function setValueName(string $valueName): CustomerOrderItemVariation
    {
        $this->valueName = $valueName;
        
        return $this;
    }
    
    /**
     * @return string
     */
    public function getValueName(): string
    {
        return $this->valueName;
    }
}
